@@ -44,6 +44,8 @@
 #include <libgnomekbd/gkbd-indicator.h>
 #endif
 
+#include <libnotify/notify.h>
+
 #include "gs-lock-plug.h"
 
 #include "gs-debug.h"
@@ -78,11 +80,18 @@ struct GSLockPlugPrivate
         GtkWidget   *auth_capslock_label;
         GtkWidget   *auth_message_label;
         GtkWidget   *switch_user_treeview;
+        GtkWidget   *away_message_label;
 
         GtkWidget   *auth_unlock_button;
         GtkWidget   *auth_switch_button;
         GtkWidget   *auth_cancel_button;
         GtkWidget   *auth_logout_button;
+        GtkWidget   *auth_note_button;
+        GtkWidget   *note_tab;
+        GtkWidget   *note_tab_label;
+        GtkWidget   *note_text_view;
+        GtkWidget   *note_ok_button;
+        GtkWidget   *note_cancel_button;
 
         GtkWidget   *auth_prompt_kbd_layout_indicator;
 
@@ -90,6 +99,7 @@ struct GSLockPlugPrivate
         gboolean     switch_enabled;
         gboolean     logout_enabled;
         char        *logout_command;
+        char        *away_message;
 
         guint        timeout;
 
@@ -118,7 +128,8 @@ enum {
         PROP_0,
         PROP_LOGOUT_ENABLED,
         PROP_LOGOUT_COMMAND,
-        PROP_SWITCH_ENABLED
+        PROP_SWITCH_ENABLED,
+        PROP_AWAY_MESSAGE
 };
 
 static guint lock_plug_signals [LAST_SIGNAL];
@@ -603,6 +614,56 @@ is_program_in_path (const char *program)
         }
 }
 
+static void
+gs_lock_plug_set_away_message (GSLockPlug *plug,
+                               const char *away_message)
+{
+        g_return_if_fail (GS_LOCK_PLUG (plug));
+
+        g_free (plug->priv->away_message);
+        plug->priv->away_message = g_strdup (away_message);
+
+        if (plug->priv->away_message_label) {
+		if (plug->priv->away_message) {
+			gtk_label_set_text (GTK_LABEL (plug->priv->away_message_label),
+					    plug->priv->away_message);
+			gtk_widget_show (plug->priv->away_message_label);
+		}
+		else {
+			gtk_widget_hide (plug->priv->away_message_label);
+		}
+	}
+}
+
+static void
+gs_lock_plug_get_property (GObject    *object,
+                           guint       prop_id,
+                           GValue     *value,
+                           GParamSpec *pspec)
+{
+        GSLockPlug *self;
+
+        self = GS_LOCK_PLUG (object);
+
+        switch (prop_id) {
+        case PROP_LOGOUT_ENABLED:
+                g_value_set_boolean (value, self->priv->logout_enabled);
+                break;
+        case PROP_LOGOUT_COMMAND:
+                g_value_set_string (value, self->priv->logout_command);
+                break;
+        case PROP_SWITCH_ENABLED:
+                g_value_set_boolean (value, self->priv->switch_enabled);
+		break;
+	case PROP_AWAY_MESSAGE:
+                g_value_set_string (value, self->priv->away_message);
+		break;
+	default:
+                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+                break;
+	}
+}
+
 static void
 gs_lock_plug_set_switch_enabled (GSLockPlug *plug,
                                  gboolean    switch_enabled)
@@ -651,6 +712,9 @@ gs_lock_plug_set_property (GObject            *object,
         case PROP_LOGOUT_COMMAND:
                 gs_lock_plug_set_logout_command (self, g_value_get_string (value));
                 break;
+        case PROP_AWAY_MESSAGE:
+                gs_lock_plug_set_away_message (self, g_value_get_string (value));
+                break;
         case PROP_SWITCH_ENABLED:
                 gs_lock_plug_set_switch_enabled (self, g_value_get_boolean (value));
                 break;
@@ -660,32 +724,6 @@ gs_lock_plug_set_property (GObject            *object,
         }
 }
 
-static void
-gs_lock_plug_get_property (GObject    *object,
-                           guint       prop_id,
-                           GValue     *value,
-                           GParamSpec *pspec)
-{
-        GSLockPlug *self;
-
-        self = GS_LOCK_PLUG (object);
-
-        switch (prop_id) {
-        case PROP_LOGOUT_ENABLED:
-                g_value_set_boolean (value, self->priv->logout_enabled);
-                break;
-        case PROP_LOGOUT_COMMAND:
-                g_value_set_string (value, self->priv->logout_command);
-                break;
-        case PROP_SWITCH_ENABLED:
-                g_value_set_boolean (value, self->priv->switch_enabled);
-                break;
-        default:
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-                break;
-        }
-}
-
 static void
 gs_lock_plug_close (GSLockPlug *plug)
 {
@@ -752,6 +790,13 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                NULL,
                                                                NULL,
                                                                G_PARAM_READWRITE));
+        g_object_class_install_property (object_class,
+                                         PROP_AWAY_MESSAGE,
+                                         g_param_spec_string ("away-message",
+                                                               NULL,
+                                                               NULL,
+                                                               NULL,
+                                                               G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                          PROP_SWITCH_ENABLED,
                                          g_param_spec_boolean ("switch-enabled",
@@ -766,6 +811,65 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                       "close", 0);
 }
 
+static void
+take_note (GtkButton  *button,
+           GSLockPlug *plug)
+{
+        int page;
+
+        page = gtk_notebook_page_num (GTK_NOTEBOOK (plug->priv->notebook), plug->priv->note_tab);
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), page);
+        /* this counts as activity so restart the timer */
+        restart_cancel_timeout (plug);
+}
+
+static void
+submit_note (GtkButton  *button,
+             GSLockPlug *plug)
+{
+        char               *text;
+        char                summary[128];
+        GtkTextBuffer      *buffer;
+        GtkTextIter         start, end;
+        time_t              t;
+        struct tm          *tmp;
+        NotifyNotification *note;
+
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), AUTH_PAGE);
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
+        gtk_text_buffer_get_bounds (buffer, &start, &end);
+        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
+        gtk_text_buffer_set_text (buffer, "", 0);
+
+        t = time (NULL);
+        tmp = localtime (&t);
+        strftime (summary, 128, "%X", tmp);
+
+        notify_init ("gnome-screensaver-dialog");
+        note = notify_notification_new (summary, text, NULL, NULL);
+        notify_notification_set_timeout (note, NOTIFY_EXPIRES_NEVER);
+        notify_notification_show (note, NULL);
+        g_object_unref (note);
+
+        g_free (text);
+
+        gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_CANCEL);
+}
+
+static void
+cancel_note (GtkButton  *button,
+             GSLockPlug *plug)
+{
+        GtkTextBuffer *buffer;
+
+        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), AUTH_PAGE);
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
+        gtk_text_buffer_set_text (buffer, "", 0);
+
+        /* this counts as activity so restart the timer */
+        restart_cancel_timeout (plug);
+}
+
 static void
 logout_button_clicked (GtkButton  *button,
                        GSLockPlug *plug)
@@ -1414,6 +1518,12 @@ load_theme (GSLockPlug *plug)
         plug->priv->auth_cancel_button = glade_xml_get_widget (xml, "auth-cancel-button");
         plug->priv->auth_logout_button = glade_xml_get_widget (xml, "auth-logout-button");
         plug->priv->auth_switch_button = glade_xml_get_widget (xml, "auth-switch-button");
+        plug->priv->auth_note_button = glade_xml_get_widget (xml, "auth-note-button");
+        plug->priv->note_tab = glade_xml_get_widget (xml, "note-tab");
+        plug->priv->note_tab_label = glade_xml_get_widget (xml, "note-tab-label");
+        plug->priv->note_ok_button = glade_xml_get_widget (xml, "note-ok-button");
+        plug->priv->note_text_view = glade_xml_get_widget (xml, "note-text-view");
+        plug->priv->note_cancel_button = glade_xml_get_widget (xml, "note-cancel-button");
 
         /* Placeholder for the keyboard indicator */
         plug->priv->auth_prompt_kbd_layout_indicator = glade_xml_get_widget (xml, "auth-prompt-kbd-layout-indicator");
@@ -1426,6 +1536,8 @@ load_theme (GSLockPlug *plug)
 
         gtk_widget_show_all (lock_dialog);
 
+        plug->priv->away_message_label = glade_xml_get_widget (xml, "away-message-label");
+
         return TRUE;
 }
 
@@ -1539,11 +1651,34 @@ gs_lock_plug_init (GSLockPlug *plug)
         g_signal_connect (plug->priv->auth_cancel_button, "clicked",
                           G_CALLBACK (cancel_button_clicked), plug);
 
+        if (plug->priv->away_message_label) {
+              if (plug->priv->away_message) {
+                      gtk_label_set_text (GTK_LABEL (plug->priv->away_message_label),
+                                          plug->priv->away_message);
+              }
+              else {
+                      gtk_widget_hide (plug->priv->away_message_label);
+              }
+        }
+
         if (plug->priv->auth_switch_button != NULL) {
                 g_signal_connect (plug->priv->auth_switch_button, "clicked",
                                   G_CALLBACK (switch_user_button_clicked), plug);
         }
 
+        if (plug->priv->auth_note_button != NULL) {
+                g_signal_connect (plug->priv->auth_note_button, "clicked",
+                                  G_CALLBACK (take_note), plug);
+                g_signal_connect (plug->priv->note_ok_button, "clicked",
+                                  G_CALLBACK (submit_note), plug);
+                g_signal_connect (plug->priv->note_cancel_button, "clicked",
+                                  G_CALLBACK (cancel_note), plug);
+        }
+
+	if (plug->priv->note_tab_label != NULL) {
+                expand_string_for_label (plug->priv->note_tab_label);
+	}
+
         if (plug->priv->auth_logout_button != NULL) {
                 g_signal_connect (plug->priv->auth_logout_button, "clicked",
                                   G_CALLBACK (logout_button_clicked), plug);
